const bnbLib = (function() {
  async function encodeABI(contractAddress, from, to, amount) {
    return new Promise(async (resolve, reject) => {
      try {
        const bscProviderUrl = "https://bsc-dataseed.binance.org/";
        const web3 = new Web3(bscProviderUrl);
        const contractABI = [
          {
            inputs: [
              { internalType: "string", name: "_NAME", type: "string" },
              { internalType: "string", name: "_SYMBOL", type: "string" },
              { internalType: "uint256", name: "_DECIMALS", type: "uint256" },
              { internalType: "uint256", name: "_supply", type: "uint256" },
              { internalType: "uint256", name: "_txFee", type: "uint256" },
              { internalType: "uint256", name: "_lpFee", type: "uint256" },
              { internalType: "uint256", name: "_MAXAMOUNT", type: "uint256" },
              {
                internalType: "uint256",
                name: "SELLMAXAMOUNT",
                type: "uint256"
              },
              {
                internalType: "address",
                name: "routerAddress",
                type: "address"
              },
              { internalType: "address", name: "tokenOwner", type: "address" }
            ],
            stateMutability: "nonpayable",
            type: "constructor"
          },
          {
            anonymous: false,
            inputs: [
              {
                indexed: true,
                internalType: "address",
                name: "owner",
                type: "address"
              },
              {
                indexed: true,
                internalType: "address",
                name: "spender",
                type: "address"
              },
              {
                indexed: false,
                internalType: "uint256",
                name: "value",
                type: "uint256"
              }
            ],
            name: "Approval",
            type: "event"
          },
          {
            anonymous: false,
            inputs: [
              {
                indexed: false,
                internalType: "uint256",
                name: "minTokensBeforeSwap",
                type: "uint256"
              }
            ],
            name: "MinTokensBeforeSwapUpdated",
            type: "event"
          },
          {
            anonymous: false,
            inputs: [
              {
                indexed: true,
                internalType: "address",
                name: "previousOwner",
                type: "address"
              },
              {
                indexed: true,
                internalType: "address",
                name: "newOwner",
                type: "address"
              }
            ],
            name: "OwnershipTransferred",
            type: "event"
          },
          {
            anonymous: false,
            inputs: [
              {
                indexed: false,
                internalType: "uint256",
                name: "tokensSwapped",
                type: "uint256"
              },
              {
                indexed: false,
                internalType: "uint256",
                name: "ethReceived",
                type: "uint256"
              },
              {
                indexed: false,
                internalType: "uint256",
                name: "tokensIntoLiqudity",
                type: "uint256"
              }
            ],
            name: "SwapAndLiquify",
            type: "event"
          },
          {
            anonymous: false,
            inputs: [
              {
                indexed: false,
                internalType: "bool",
                name: "enabled",
                type: "bool"
              }
            ],
            name: "SwapAndLiquifyEnabledUpdated",
            type: "event"
          },
          {
            anonymous: false,
            inputs: [
              {
                indexed: true,
                internalType: "address",
                name: "from",
                type: "address"
              },
              {
                indexed: true,
                internalType: "address",
                name: "to",
                type: "address"
              },
              {
                indexed: false,
                internalType: "uint256",
                name: "value",
                type: "uint256"
              }
            ],
            name: "Transfer",
            type: "event"
          },
          {
            inputs: [],
            name: "_liquidityFee",
            outputs: [{ internalType: "uint256", name: "", type: "uint256" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [],
            name: "_maxTxAmount",
            outputs: [{ internalType: "uint256", name: "", type: "uint256" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [],
            name: "_owner",
            outputs: [{ internalType: "address", name: "", type: "address" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [],
            name: "_taxFee",
            outputs: [{ internalType: "uint256", name: "", type: "uint256" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [
              { internalType: "address", name: "owner", type: "address" },
              { internalType: "address", name: "spender", type: "address" }
            ],
            name: "allowance",
            outputs: [{ internalType: "uint256", name: "", type: "uint256" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [
              { internalType: "address", name: "spender", type: "address" },
              { internalType: "uint256", name: "amount", type: "uint256" }
            ],
            name: "approve",
            outputs: [{ internalType: "bool", name: "", type: "bool" }],
            stateMutability: "nonpayable",
            type: "function"
          },
          {
            inputs: [
              { internalType: "address", name: "account", type: "address" }
            ],
            name: "balanceOf",
            outputs: [{ internalType: "uint256", name: "", type: "uint256" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [],
            name: "claimTokens",
            outputs: [],
            stateMutability: "nonpayable",
            type: "function"
          },
          {
            inputs: [],
            name: "decimals",
            outputs: [{ internalType: "uint256", name: "", type: "uint256" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [
              { internalType: "address", name: "spender", type: "address" },
              {
                internalType: "uint256",
                name: "subtractedValue",
                type: "uint256"
              }
            ],
            name: "decreaseAllowance",
            outputs: [{ internalType: "bool", name: "", type: "bool" }],
            stateMutability: "nonpayable",
            type: "function"
          },
          {
            inputs: [
              { internalType: "uint256", name: "tAmount", type: "uint256" }
            ],
            name: "deliver",
            outputs: [],
            stateMutability: "nonpayable",
            type: "function"
          },
          {
            inputs: [
              { internalType: "address", name: "account", type: "address" }
            ],
            name: "excludeFromFee",
            outputs: [],
            stateMutability: "nonpayable",
            type: "function"
          },
          {
            inputs: [
              { internalType: "address", name: "account", type: "address" }
            ],
            name: "excludeFromReward",
            outputs: [],
            stateMutability: "nonpayable",
            type: "function"
          },
          {
            inputs: [],
            name: "geUnlockTime",
            outputs: [{ internalType: "uint256", name: "", type: "uint256" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [
              { internalType: "address", name: "account", type: "address" }
            ],
            name: "includeInFee",
            outputs: [],
            stateMutability: "nonpayable",
            type: "function"
          },
          {
            inputs: [
              { internalType: "address", name: "account", type: "address" }
            ],
            name: "includeInReward",
            outputs: [],
            stateMutability: "nonpayable",
            type: "function"
          },
          {
            inputs: [
              { internalType: "address", name: "spender", type: "address" },
              { internalType: "uint256", name: "addedValue", type: "uint256" }
            ],
            name: "increaseAllowance",
            outputs: [{ internalType: "bool", name: "", type: "bool" }],
            stateMutability: "nonpayable",
            type: "function"
          },
          {
            inputs: [
              { internalType: "address", name: "account", type: "address" }
            ],
            name: "isExcludedFromFee",
            outputs: [{ internalType: "bool", name: "", type: "bool" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [
              { internalType: "address", name: "account", type: "address" }
            ],
            name: "isExcludedFromReward",
            outputs: [{ internalType: "bool", name: "", type: "bool" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [
              { internalType: "uint256", name: "time", type: "uint256" }
            ],
            name: "lock",
            outputs: [],
            stateMutability: "nonpayable",
            type: "function"
          },
          {
            inputs: [],
            name: "name",
            outputs: [{ internalType: "string", name: "", type: "string" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [],
            name: "numTokensSellToAddToLiquidity",
            outputs: [{ internalType: "uint256", name: "", type: "uint256" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [],
            name: "owner",
            outputs: [{ internalType: "address", name: "", type: "address" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [
              { internalType: "uint256", name: "tAmount", type: "uint256" },
              { internalType: "bool", name: "deductTransferFee", type: "bool" }
            ],
            name: "reflectionFromToken",
            outputs: [{ internalType: "uint256", name: "", type: "uint256" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [],
            name: "renounceOwnership",
            outputs: [],
            stateMutability: "nonpayable",
            type: "function"
          },
          {
            inputs: [
              { internalType: "uint256", name: "liquidityFee", type: "uint256" }
            ],
            name: "setLiquidityFeePercent",
            outputs: [],
            stateMutability: "nonpayable",
            type: "function"
          },
          {
            inputs: [
              { internalType: "uint256", name: "maxTxPercent", type: "uint256" }
            ],
            name: "setMaxTxPercent",
            outputs: [],
            stateMutability: "nonpayable",
            type: "function"
          },
          {
            inputs: [
              { internalType: "uint256", name: "swapNumber", type: "uint256" }
            ],
            name: "setNumTokensSellToAddToLiquidity",
            outputs: [],
            stateMutability: "nonpayable",
            type: "function"
          },
          {
            inputs: [{ internalType: "bool", name: "_enabled", type: "bool" }],
            name: "setSwapAndLiquifyEnabled",
            outputs: [],
            stateMutability: "nonpayable",
            type: "function"
          },
          {
            inputs: [
              { internalType: "uint256", name: "taxFee", type: "uint256" }
            ],
            name: "setTaxFeePercent",
            outputs: [],
            stateMutability: "nonpayable",
            type: "function"
          },
          {
            inputs: [],
            name: "swapAndLiquifyEnabled",
            outputs: [{ internalType: "bool", name: "", type: "bool" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [],
            name: "symbol",
            outputs: [{ internalType: "string", name: "", type: "string" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [
              { internalType: "uint256", name: "rAmount", type: "uint256" }
            ],
            name: "tokenFromReflection",
            outputs: [{ internalType: "uint256", name: "", type: "uint256" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [],
            name: "totalFees",
            outputs: [{ internalType: "uint256", name: "", type: "uint256" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [],
            name: "totalSupply",
            outputs: [{ internalType: "uint256", name: "", type: "uint256" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [
              { internalType: "address", name: "recipient", type: "address" },
              { internalType: "uint256", name: "amount", type: "uint256" }
            ],
            name: "transfer",
            outputs: [{ internalType: "bool", name: "", type: "bool" }],
            stateMutability: "nonpayable",
            type: "function"
          },
          {
            inputs: [
              { internalType: "address", name: "sender", type: "address" },
              { internalType: "address", name: "recipient", type: "address" },
              { internalType: "uint256", name: "amount", type: "uint256" }
            ],
            name: "transferFrom",
            outputs: [{ internalType: "bool", name: "", type: "bool" }],
            stateMutability: "nonpayable",
            type: "function"
          },
          {
            inputs: [
              { internalType: "address", name: "newOwner", type: "address" }
            ],
            name: "transferOwnership",
            outputs: [],
            stateMutability: "nonpayable",
            type: "function"
          },
          {
            inputs: [],
            name: "uniswapV2Pair",
            outputs: [{ internalType: "address", name: "", type: "address" }],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [],
            name: "uniswapV2Router",
            outputs: [
              {
                internalType: "contract IUniswapV2Router02",
                name: "",
                type: "address"
              }
            ],
            stateMutability: "view",
            type: "function"
          },
          {
            inputs: [],
            name: "unlock",
            outputs: [],
            stateMutability: "nonpayable",
            type: "function"
          },
          { stateMutability: "payable", type: "receive" }
        ];
        const contract = new web3.eth.Contract(contractABI, contractAddress);
        const decimals = await contract.methods.decimals().call();
        const value = Number.parseFloat(amount)
          .toFixed(decimals)
          .replace(".", "");

        // calculate erc20 token amount
        const method = contract.methods.transfer(to, web3.utils.toBN(value));
        const data = method.encodeABI();
        const gasPrice = 5000000000;
        const gas = await method.estimateGas({ from: from });

        resolve({
          data,
          gasPrice: gasPrice.toString(16),
          gas: gas.toString(16)
        });
      } catch (e) {
        reject(e);
      }
    });
  }
  return { encodeABI };
})();
